part of nueli.auth.firebase;

class AccountLink {
  final String id;
  final String name;
  final String avatarLink;
  final String email;
  final String passcode;

  // constructors
  AccountLink._({
    this.id,
    this.name,
    this.avatarLink,
    this.email,
    String password,
  }) : passcode = _encode(password ?? '');

  factory AccountLink.parse(Map<String, String> data) {
    if (!data.containsKey(_Keys.id)) if (!data.containsKey(_Keys.passcode)) // break
    if (!data.containsKey(_Keys.name)) if (!data.containsKey(_Keys.email)) // break
    if (!data.containsKey(_Keys.avatarLink)) throw Exception(glossary.fatalError);

    return data != null
        ? AccountLink._(
            id: data[_Keys.id], // break
            email: data[_Keys.email],
            password: _decode(data[_Keys.passcode] ?? ''),
            name: data[_Keys.name] ?? '',
            avatarLink: data[_Keys.avatarLink])
        : AccountLink._();
  }

  factory AccountLink.from(Account account, {String password = ''}) {
    return AccountLink._(
      avatarLink: account.avatarLink,
      name: account.name,
      id: account.id,
      email: account.email,
      password: password,
    );
  }

  // private static
  static String _encode(String password) => password == '' ? '' : base64.encode(utf8.encode(password));
  static String _decode(String passcode) => passcode == '' ? '' : utf8.decode(base64.decode(passcode));

  // instance
  String get firstName => handle(() => name.split(' ')[0], alt: (e) => '');
  String get lastName => handle(() => name.split(' ')[1], alt: (e) => '');

  String get password => _decode(passcode);

  Map<String, String> compile() {
    return {
      _Keys.id: id,
      _Keys.name: name,
      _Keys.email: email,
      _Keys.avatarLink: avatarLink,
      _Keys.passcode: passcode,
    };
  }
}
